/* egprs_rlc_compression.h
 *  Routines for EGPRS RLC bitmap compression handling
 */

#pragma once

#define BITS_TO_BYTES(X)	(((X) ? (X)/8 : 0) + 1)
#define MOD8(X)			((X) & (0x07))
#define  MOD64(X)	((X + 64) & 0x3F)

typedef struct egprs_compress_node{
	struct egprs_compress_node *left;
	struct egprs_compress_node *right;
	int run_length;
} egprs_compress_node;

int decompress_crbb(int8_t compress_bmap_len, uint8_t clr_code_bit,
			const uint8_t *orig_buf, bitvec *dest);
int osmo_t4_compress(struct bitvec *bv);
int compress_rbb(
		struct bitvec *urbb_vec,/*!< input bitvector to compress */
		struct bitvec *crbb_vec,  /*!< Compressed bitmap len */
		uint8_t *uclen_crbb,  /*!< Uncompressed bitmap len in CRBB */
		uint8_t  max_bits     /*!< Maximum remaining bits */
		);

/* Singleton to manage the EGPRS compression algorithm. */
class egprs_compress
{
public:
	static int decompress_crbb(int8_t compress_bmap_len,
		bool color_code_bit, const uint8_t *orig_buf,
		bitvec *dest);

private:
	static egprs_compress *s_instance;
	egprs_compress_node *ones_list;
	egprs_compress_node *zeros_list;

	void decode_tree_init(void);

	static egprs_compress *instance()
	{
		if (!s_instance)
			s_instance = new egprs_compress;

		return s_instance;
	}

	egprs_compress()
	{
		decode_tree_init();
	}
	egprs_compress_node *create_tree_node(void *);
	void build_codewords(egprs_compress_node *root, const char *cdwd[]);
	/* singleton class, so this private destructor is left unimplemented. */
	~egprs_compress();
};
